function phasordata=acquire(params,level,duration,calfile,datafile)
% acquire(params,level,duration,calfile,datafile) plays sine-wave bursts 
% at a sequence of frequencies, records each
% microphone channel at each frequency, finds the phasor (complex amplitude) 
% for each channel and frequency, and calibrates the phasor data using
% the calibration file calfile.mat, which must be present in the current
% directory.  The frequency list is is taken from the calibration file.
% The output data structure phasordata is written to the
% file datafile.mat in the current directory.
%
% Inputs:
% params          parameters data structure, see defaults.m for details
% level           burst amplitude
% duration        burst duration in seconds, including 5% transitions
% calfile         calibration file name (without file extension),
%                 must be a string          
% datafile        file name for results (without file extension),
%                 must be a string
%
% Output:
% phasordata.phasors   Complex amplitudes for each frequency and channel
%                      Number of rows = size(freqs,1)
%                      Number of columns = params.nchan
% phasordata.freqs     Column vector of nf frequencies.
% phasordata.params    The complete parameters data structure
% phasordata.level     burst amplitude
% phasordata.duration  burst duration in seconds
% phasordata.calfile   name of calibration file (string)
% phasordata.timestamp time of creation of output file
%
% The acquire function writes a file in the current directory
% with the name datafile.mat that contains the phasordata structure.  If
% datafile.mat already exists in the current directory, acquire returns
% without doing anything.  It also returns without doing anything if
% the calibration file calfile.mat is not present in the current directory.
%
% Uses: sweep

% Check to see if calfile.mat exists in the current directory
calpath=[cd '\' calfile '.mat'];
resultcode=exist(calpath,'file'); 
if resultcode ~= 2;  % if not= 2 then file is not present
    error([calfile '.mat cannot be found in current directory']);
else
    load(calpath)    % if calfile.mat exists, load the structure 
                     % caldata into the workspace
end

% Check that output file is not already present in current directory.
outpath=[cd '\' datafile '.mat'];
resultcode=exist(outpath,'file'); 
if resultcode == 2;  % if = 2 then file is already present
    error([datafile '.mat already exists in current directory']);
end

% Check that the number of channels in the cal file is the same as the 
% number of channels specified in the parameters structure.  
if caldata.params.nchan ~= params.nchan
   error('number of channels does not agree with calibration file');
end

% Get frequency list from calibration file
freqs=caldata.freqs;

% Acquire data.  Sweep returns fields .phasors and .freqs.
phasordata=sweep(params,level,duration,freqs);

% Normalize the calibration phasors to 1st channel to remove level variations
% that come from the calibration.  
normcaldata=caldata.phasors ./ (repmat(caldata.phasors(:,1),1,params.nchan));

% Calibrate the data.  All caldata.phasors entries must be nonzero.
phasordata.phasors=phasordata.phasors ./ normcaldata;

% Pass information to the phasordata structure
phasordata.params=params;
phasordata.level=level;
phasordata.duration=duration;
phasordata.calfile=calfile;
phasordata.timestamp=datestr(now);

% Write phasordata structure to file datafile.mat.  
save(datafile,'phasordata');
