function timeseries=burst(params,level,duration,frequency)
% timeseries=burst(params,level,duration,frequency) returns a sine wave
% burst time-series.  Sine-squared envelope transitions are used to avoid
% clicks.  Rise and fall transitions are each 5% of total duration.
%
% Inputs:
% params          Parameters data structure, see defaults.m for details.
% level           Amplitude of sine wave in steady state region
% duration        Total duration in seconds
% frequency       Frequency
%
% Outputs:
% timeseries.samples    Column vector of samples, number of samples is
%                       equal to round(dur*params.fsamp).
% timeseries.times      Column vector containing the time of each
%                       sample, starting at time=0.

fsamp=params.fsamp;            % sampling frequency
omega = 2*pi*frequency;        % angular frequency
trans = 0.05*duration;         % rise and fall time
omegatrans = 2*pi/(4*trans);   % angular freq. for rise and fall envelope

nsamp = round(duration*fsamp);    % total number of samples
nsamptrans = round(trans*fsamp);  % number of rise and fall samples
nsampflat = nsamp-2*nsamptrans;   % number of samples between rise and fall

% create series of sample times
risetime = (0:nsamptrans-1)/fsamp;
flattime = (nsamptrans:nsamptrans+nsampflat-1)/fsamp;
falltime = (nsamptrans+nsampflat:nsamp-1)/fsamp;
timeseries.times = [risetime, flattime, falltime]';

% create output time series with sin^2 rise and fall envelope
envelope = [sin(omegatrans*risetime).^2,ones(1,size(flattime,2)),cos(omegatrans*risetime).^2];
timeseries.samples = ((level*sin(omega*timeseries.times+pi/2)) .* envelope');
