function caldata=calibrate(params,level,duration,firstf,lastf,nf,calfile)
% calibrate(params,level,duration,firstf,lastf,nf,calfile) calibrates 
% microphone phase and amplitude.  calibrate.m plays a sine-wave burst 
% at a sequency of frequencies, records each microphone channel, and 
% finds the phasor (complex amplitude) for each microphone and frequency.
% 
% Microphones should all be mounted at the same reference plane.
% Let hardware warm up and stabilize before calibration.
% Microphone gains must not be changed after calibration unless
% relative gains can be preserved.
%
% Inputs:
% params          parameters data structure, see defaults.m for details
% level           burst amplitude
% duration        burst duration in seconds, including 5% transitions
% firstf          first frequency
% lastf           last frequency
% nf              number of frequencies
% calfile         calibration file name (without file extension),
%                 must be a string          
%
% Output:
% caldata.phasors    Complex amplitudes for each frequency and channel
%                    Number of rows = size(freqs,1)
%                    Number of columns = params.nchan
% caldata.freqs      Column vector of nf frequencies.
% caldata.params     The complete parameters data structure
% caldata.level      burst amplitude
% caldata.duration   burst duration in seconds
% caldata.timestamp  time of creation of output file
%
% The calibrate function writes a file in the current directory
% with the name contained in the variable calfile. The output file
% contains the caldata structure.  If the output file
% already exists in the current directory, calibrate reports and error
% and returns without doing anything.  
%
% Uses: sweep

% Check that output file is not already present in current directory.
outpath=[cd '\' calfile '.mat'];
resultcode=exist(outpath,'file'); 
if resultcode == 2;  % if = 2 then file is already present
    error([calfile '.mat already exists in current directory']);
end

% Create frequency array.
% For linearly spaced frequencies use this line instead:
% freq=linspace(firstf,lastf,nf)
freqs=logspace(log10(firstf),log10(lastf),nf)';

caldata=sweep(params,level,duration,freqs);
caldata.params=params;
caldata.level=level;
caldata.duration=duration;
caldata.timestamp=datestr(now);

% Write caldata structure to file calfile.mat.  
save(calfile,'caldata');
