function phasors=fourier(params,timeseries,frequency)
% phasors=fourier(params,timeseries,frequency) computes the complex amplitudes
% (phasors) of a multi-channel time series at the specified frequency. 
% This routine is used to make accurate complex amplitude measurements at
% an exactly known frequency.  Before computing the amplitudes, the first 
% and last 10% of the time series is removed to allow for settling of 
% transients.  The time series can have any number of synchronous channels.
% The overall phase varies with the play latency time and so generally does not
% contain useful information.  However, recording is synchronized
% from channel to channel so the relative phases are significant.
%
% Inputs:
% params                  Parameters data structure, see defaults.m
%                         for details.
% timeseries.samples      Time-series samples with one column for 
%                         each channel.  Any number of channels is
%                         allowed.
% timeseries.times        Column vector containing the time of each
%                         sample, starting at time=0.
% frequency               frequency at which the complex amplitudes are
%                         determined.
%
% Outputs:
% phasors                 Row vector containing the complex amplitude
%                         of each channel.  The number of complex
%                         amplitudes is equal to the number of channels
%                         in the time series.
%
% Note: sampling rate of the input time series is assumed to be
% params.fsamp.  It is not found from timeseries.times.

nsamp=size(timeseries.samples,1);   % find number of samples in timeseries
nchan=size(timeseries.samples,2);   % find number of channels in timeseries
ntrans=round(nsamp/10);             % number of transient samples to remove
fsamp=params.fsamp;                 % get sampling rate

% Remove twice the transient time from both ends of the
% time series.

cleanseries = timeseries.samples(ntrans+1:end-ntrans,:);
nsampclean = nsamp-2*ntrans;

% Measure the complex amplitudes.  First, trim time series so it contains, 
% as nearly as possible, an integer number of cycles at the known 
% measurement frequency.  Then integrate times a complex exponential at
% the measurement frequency.  This algorithm requires no relationship
% between the measurement frequency and the sampling frequency, and it
% has no errors from the 'picket fence effect'.  It will
% not be compromised by higher harmonics in the time series.

nsamptrim=floor( (fsamp/frequency)*floor((nsampclean-1)*(frequency/fsamp)) )+1;

time = (0:nsamptrim-1)/fsamp;

arg =  2*pi*frequency*time;
argmatrix = repmat(arg',1,nchan);

% for exp(+iwt) time dependence, transform with exp(-iwt)
phasors = sum( (2.0/nsamptrim)*cleanseries(1:nsamptrim,:).*exp(-1i*argmatrix), 1 );
