function F=wavefcn3(Adj,freqs,x)  
% wavefcn3(Adj,freqs,x) is a wavefunction for the lsqnonlin.m least 
% squares nonlinear fitting routine.  Adj contains all of the 
% parameters that will be adjusted to make the phasors returned by 
% wavefcn3 match the phasor data as closely as possible.  This version
% models the propagation constant with three parameters (see calling
% function for details).
% 
% wavefcn3 returns modeled phasors labeled with 3 indices.  The first index 
% runs over the frequencies in freqs, and the second runs over the mic 
% positions in x.  The last index is for real and imaginary parts. freqs
% and Adj are column vectors while x is a row vector.
% 
% Subtract phasor data from the matrix returned by wavefcn3 and pass the 
% resulting residual matrix to lsqnonlin.m.  lsqnonlin will square and 
% sum the elements and then minimize with respect to Adj, which must 
% be a real vector. lsqnonlin only works with real variables so a third 
% index is used to distinquish real and imaginary parts of phasors.
% See calling routine for use of anonymous function to pass coordinate
% values to the cost function.  

% If nfreq is the number of frequency points:
%     the first nfreq elements of Adj are the right-going real-part wave amps to fit
%     the second nfreq elements of Adj are the right-going imag-part wave amps to fit
%     the third nfreq elements of Adj are the left-going real-part wave amps to fit
%     the fourth nfreq elements of Adj are the left-going imag-part wave amps to fit
%     the next element of Adj is the global free space sound velocity
%     the next element of Adj is the coeff. of a sqrt(omega) contribution
%       to the attenuation
%     the last element is the 1/sqrt(omega) coeff. of the phase velocity

nfreq=size(freqs,1);  % find number of frequency points
nchan=size(x,2);      % find number of x points
omega=2*pi*freqs;

% unpack amplitude parameters to fit
AmpRR=Adj(1:nfreq);           %  right-going real part
AmpRI=Adj(nfreq+1:2*nfreq);   %  right-going imaginary part
AmpLR=Adj(2*nfreq+1:3*nfreq); %  left-going real part
AmpLI=Adj(3*nfreq+1:4*nfreq); %  left-going imaginary part

vs=Adj(4*nfreq+1);      % unpack global free space sound velocity
beta=Adj(4*nfreq+2);    % unpack global atten. sqrt(omega) coeff.
delta=Adj(4*nfreq+3);   % unpack phase velocity coeff.

atten=beta*sqrt(omega);         % construct attenuation vector
vphase=vs*(1.0-vs*delta./sqrt(omega)); % phase velocity vector
k=omega./vphase;                       % construct wave number vector

AmpRRmat=repmat(AmpRR,1,nchan); % Expand real and imag. parts of amplitudes
AmpRImat=repmat(AmpRI,1,nchan); % to cover whole coordinate space.
AmpLRmat=repmat(AmpLR,1,nchan); % First index runs over omega, second over
AmpLImat=repmat(AmpLI,1,nchan); % positions.
                                % These matricies are independent of
                                % position.

% Compute cos(k*x), sin(k*x), exp(atten*x), exp(-atten*x) matricies.
% First index runs over omega, second over x.
cs=cos(kron(k,x));
sn=sin(kron(k,x));
explus=exp(kron(atten,x));
exminus=exp(-kron(atten,x));

% Compute modeled phasors at each frequency and position.
% Third index is real,imag. part.

F(:,:,1)=(AmpRRmat.*cs+AmpRImat.*sn).*exminus +... 
         (AmpLRmat.*cs-AmpLImat.*sn).*explus ;

F(:,:,2)=(AmpRImat.*cs-AmpRRmat.*sn).*exminus +... 
         (AmpLImat.*cs+AmpLRmat.*sn).*explus ;
