function refl=reflect(wavefile,reflfile)
% reflect(wavefile,reflfile) calculates the reflection coefficient 
% and plots it, both on the complex plane and as magnitude and phase versus
% frequency.  The input file must contain a wave data structure, 
% as generated by wavefit.m.  The string wavefile is the input 
% file name.  This file must be present in the current directory.
%
% When the plots have been displayed the user is prompted to type a
% new value for the reference plane location, after which the
% plots are updated.  This continues until the user types 'q'
% to quit or 's' to save the refl data structure to a file
% named reflfile.  The file name reflfile is an optional input
% parameter.  If it is specified, the file it names must not already be
% present in the current directory.
%
% Inputs:
% wavefile        input file name (without file extension),
%                 must be a string 
% reflfile        output file name (without file extension),
%                 must be a string
% 
% Output:
% refl.reflcoef   Complex reflection coefficient, a column vector
%                 with one entry for each frequency
% refl.freqs      Column vector of frequencies
% refl.refplane   Location of the reference plane, measured in meters
%                 from the fiducial plane.
% refl.wavefile   The input file name, a string.
%
% An error is generated if the input file does not exist in 
% the current directory, or if the output file already exists.
%
% Uses: reflfcn.m

% Check to see if wavefile.mat exists in the current directory
inpath=[cd '\' wavefile '.mat'];
resultcode=exist(inpath,'file'); 
if resultcode ~= 2;  % if not= 2 then file is not present
   error([wavefile '.mat cannot be found in current directory']);
else            % load the wave data structure into current workspace
   load(inpath);
end

% Check that output file is not already present in current directory,
% but only if there are two input arguments.
if nargin == 2;  
outpath=[cd '\' reflfile '.mat'];
resultcode=exist(outpath,'file'); 
if resultcode == 2;  % if = 2 then file is already present
    error([reflfile '.mat already exists in current directory']);
end
end

% Initialize reference plane location, distance from fiducial 
% plane to reference plane in meters.
refplane=0.5690;

% Create magnitude and phase figure and get handle
MandP=figure('Name',...
 ['Reflection coefficient magnitude and phase: ' wavefile '.mat']); 

% Create complex plane figure and get handle
Cmplx=figure('Name',...
 ['Reflection coefficient on complex plane: ' wavefile '.mat']);

% find axes that will show all data
reflcoef=reflfcn(wave.Ramp,wave.Lamp,...
       wave.vphase,wave.atten,wave.freqs,refplane);
scale=max(abs(reflcoef));
reflaxes = 1.1*scale*[-1,1,-1,1];
% reflaxes = [-1.1,1.1,-1.1, 1.1];  % initial axes for reflection plane
% reflaxes = [-3.1,3.1,-3.1, 3.1];

% Start loop, continue until 'q' or 's' is typed
flag = true;   
while flag == true
    
% compute reflection coefficient
reflcoef=reflfcn(wave.Ramp,wave.Lamp,...
       wave.vphase,wave.atten,wave.freqs,refplane);

% Plot magnitude of reflection coefficient.
figure(MandP)        % make magnitude and phase figure current
subplot(2,1,1), plot(wave.freqs,abs(reflcoef))
xlabel('frequency (Hz)'), grid on
ylabel('magnitude of refl. coef.')

subplot(2,1,2), plot(wave.freqs,unwrap(angle(reflcoef))) 
xlabel('frequency (Hz)'), grid on
ylabel('phase of refl. coef.')

% Plot reflection coefficient on complex plane.
figure(Cmplx)       % make complex plane figure current
plot(reflcoef)
axis(reflaxes)      % set axes
xlabel('reflection plane'), grid on
% label plot with the current reference plane location
location=...        % find good location for text
  [reflaxes(1)+(reflaxes(2)-reflaxes(1))/10,...
         reflaxes(3)+(reflaxes(4)-reflaxes(3))/10];
text(location(1),location(2),['reference plane: ' num2str(refplane) ' m'])  

% fill data structure refl
refl.reflcoef=reflcoef;
refl.freqs=wave.freqs;
refl.refplane=refplane;
refl.wavefile=wavefile;

% Let user change the reference plane location
cmd = input('Ref. plane, ''q'' to quit, or ''s'' to save:','s')';
% Try to convert string from keyboard into a number.
% str2double sometimes returns errors, so catch them and return NaN.
try
   cmdnum=str2double(cmd);
catch
   cmdnum=NaN;
end

if strcmp(cmd,'q')     % quit
   return
elseif strcmp(cmd,'s') % save to output file if it exists
   if nargin == 2 
       save(reflfile,'refl')
       return
   else
       display('No output file was specified.')
   end
elseif ~isnan(cmdnum);          % was a number typed?
   refplane=real(cmdnum(1));    % if yes, change reference plane
else
   display('Command not recognized');
end

% before looping, save refl. plane axes so they will be sticky
figure(Cmplx)       % make complex plane figure current
reflaxes=axis;

end

