% scope.m script plays and records sine wave test bursts.  It is used for
% testing the hardware, setting gains, measuring the play latency, and 
% checking for harmonic distortion.
%
% 1.  Create a sin wave burst using sin^2 transition envelopes to 
%       avoid clicks.
% 2.  Play the burst and record all channels.
% 3.  Plot played and recorded time series.
% 4.  Compute magnitude spectrum with flat-top window and plot
% 5.  Accept any command from keyboard
% 6.  Repeat until 'q' is typed
% 
% Accepts any MATLAB command at keyboard prompt. Separate multiple 
% commands by commas. The burst parameters can be changed at the 
% keyboard prompt:
% 
% freq           burst frequency in Hz
% level          burst amplitude (dimensionless, converted to voltage
%                                 and sound pressure by hardwave)
% dur            burst duration in seconds
%
% For example, type 'freq=2000' to change the burst frequency to 2 kHz.
%
% Type 'r' to rescale axes and show all data in plots.
% Type 'q' to quit loop.
%
% All fields in the parameters data structure can also be changed at 
% the keyboard prompt.  For example, type 'params.latency=0.030' to 
% change the latency parameter to 30 ms, or type 'params=defaults()' to
% return the parameters to default values.  See defaults.m for details
% on the parameters data structure.
%
% The latency parameter should be adjusted to align the plots of the 
% played and recorded time series.
%
% All changes are sticky while scope.m is running but return to 
% defaults at start.  To change starting values of freq, amp or dur,
% edit this m-file.  To change default values in the parameters data 
% structure, edit the file defaults.m.

% Uses:  defaults.m, burst.m, playrecord.m, spectrum.m

% starting values
params=defaults(); % create parameters structure with default values

level=0.8;         % burst amplitude
dur=1.0;           % burst duration in seconds
freq =1000;        % burst frequency

% initial axis limits for plots
Vplay = [0,dur, -1.1*level, 1.1*level];
Vrec =  [0,dur, -1.1, 1.1];
% spectrum plot range is -150 dB to 0 dB  (0 dB=amplitude of 1.0)
Vspect =[0,params.fsamp/2,-150,0];  
% create figure for plots and get a handle to it
fighandle=figure('Name','Scope');

% start loop, continue until 'q' is typed
flag = true;   
while flag == true

% get burst time series
playseries=burst(params,level,dur,freq);

% play burst and record mic channels, time-shifted for play latency
recordseries=playrecord(params,playseries);

% make the Scope figure current so it will be the target for plots
figure(fighandle);

% plot the played time series
subplot(3,1,1), plot(playseries.times,playseries.samples)
axis(Vplay)     % set axis limits
xlabel('time (s)'), grid on
ylabel('played time series')

% plot the recorded times series, all channels
subplot(3,1,2), plot(recordseries.times,recordseries.samples)
axis(Vrec)      % set axis limits
xlabel('time (s)'), grid on
ylabel('recorded time series')

% compute flat-top windowed spectrum of recorded time series
spectseries=spectrum(params,recordseries);

% plot spectrum, all channels
subplot(3,1,3), plot(spectseries.freqs,20*log10(spectseries.mags))
axis(Vspect)   % set axis limits
ylabel('magnitude (dB)'), grid on
xlabel('frequency (Hz)')

% get command from keyboard
cmdstring = input('Any command or ''q'' to quit, ''r'' to rescale: ','s')';

  if cmdstring == 'q'          %  if true done looping
      return
  elseif cmdstring == 'r'      %  if true rescale plot axes
      Vplay = [0,dur, -1.1*level, 1.1*level];  %  show all data
      subplot(3,1,1)
      axis(Vplay)              %  set axes
      Vrec =  [0,dur, -1.1, 1.1];
      subplot(3,1,2)
      axis(Vrec)
      Vspect = [0,params.fsamp/2,-150,0];
      subplot(3,1,3)
      axis(Vspect)
  else       % execute any command from keyboard, catch errors
      evalin('base',cmdstring,'display(''Not a valid command'')')
  end

figure(fighandle);  % make scope figure current
subplot(3,1,1)      % get current axis limits before looping
Vplay=axis;
subplot(3,1,2)
Vrec=axis;
subplot(3,1,3)
Vspect=axis;

end