function spectseries=spectrum(params,timeseries)
% spectseries=spectrum(params,timeseries) computes the magnitude spectrum
% of a time series using the fft and a flat-top window.  This routine is 
% intended to measure amplitudes of recorded sine waves
% with good accuracy.  No phase information is provided and
% frequency resolution is relatively poor.  Useful for setting
% gains and checking for harmonic distortion.  Before computing the 
% spectrum, the first and last 10% of the time series is removed 
% to allow for settling of transients.  The time series can have any
% number of synchronous channels.
%
% Inputs:
% params                  Parameters data structure, see defaults.m
%                         for details.
% timeseries.samples      Time-series samples with one column for 
%                         each channel.  Any number of channels is
%                         allowed.
% timeseries.times        Column vector containing the time of each
%                         sample, starting at time=0.
%
% Outputs:
% spectseries.mags        Column vectors of measured amplitudes with
%                         one column for each channel in timeseries.
% spectseries.freqs       Column vector of frequencies for plotting 
%                         the spectrum.
%
% Note: sampling rate of the input time series is assumed to be
% params.fsamp.  It is not found from timeseries.times.

nsamp=size(timeseries.samples,1);   % find number of samples in timeseries
nchan=size(timeseries.samples,2);   % find number of channels in timeseries
ntrans=round(nsamp/10);             % number of transient samples to remove

% Remove twice the transient time from both ends of the
% recorded signal so that fft sees only the steady state.
% This is essential for amplitude accuracy.

trimmedseries = timeseries.samples(ntrans+1:end-ntrans,:);
nsamptrimmed = nsamp-2*ntrans;

% Adjust for even number of samples.  This centers the last
% frequency bin exactly at the Nyquist frequency params.fsamp/2
nsampeven=nsamptrimmed;
if floor(nsamptrimmed/2)~=nsamptrimmed/2;   %true if odd
    nsampeven=nsamptrimmed-1;
end

% flat-top window the time series
win=flattopwin(nsampeven);  
winmatrix=repmat(win,1,nchan);
windat=winmatrix.*trimmedseries(1:nsampeven,:);

% compute fft spectrum
spect = (2.0/nsampeven)*fft(windat);  
% tweak the amplitude calibration for best accuracy at 1 kHz and 
% fsamp=44100.  The constant was set empirically.
spectseries.mags = abs( spect(1:((nsampeven/2)+1),:) )/0.215595133;   

% create array of frequencies
spectseries.freqs = (0:nsampeven/2)'*(params.fsamp/nsampeven);