function phasorseries=sweep(params,level,duration,freqs)
% phasorseries=sweep(params,level,duration,freqs) plays a sine wave
% burst at each frequency, records multiple channels, and returns a complex
% phasor for each frequency and channel.  The overall phase at each
% frequency varies with the play latency time and so generally does not
% contain useful information.  However, recording is synchronized
% from channel to channel so the relative phases are significant.
%
% Inputs:
% params          Parameters data structure, see defaults.m for details.
% level           Amplitude of played sine wave
% duration        Duration of each burst in seconds
% freqs           Column vector of frequencies
%
% Outputs:
% phasorseries.phasors    Complex amplitudes for each frequency and channel
%                         Number of rows = size(freqs,1)
%                         Number of columns = params.nchan
% phasorseries.freqs          Column vector of frequencies.
%                             Same as input freqs column vector.
%
% Uses: burst, playrecord, fourier

% reserve space for output
nfreqs=size(freqs,1);
phasorseries.phasors=zeros(nfreqs,params.nchan);
phasorseries.freqs=freqs;

% loop over frequencies
%   burst creates the sine wave timeseries
%   playrecord plays the sine wave and records each channel
%   fourier finds the phasor (complex amplitude) for each recorded channel
for n=1:nfreqs
    playseries=burst(params,level,duration,freqs(n));
    recordseries=playrecord(params,playseries);
    phasorseries.phasors(n,:)=fourier(params,recordseries,freqs(n));
end