function viewdata(filename1,filename2)
% viewdata(filename1,filename2) creates plots for inspecting data
% files, both those containing the caldata structure created by
% calibrate.m, and those containing the phasordata structure created
% by acquire.m.  Either one or two file names may be given.  
% Each plot contains one curve for each channel, distinguished by color.
% For some plots the phasor data is normalized at each frequency
% by dividing the phasor for each channel by the 1st phasor.  This
% removes any magnitude variations that are common to all channels and also 
% removes the overall phase, which generally contains no information
% and varies randomly from one frequency to the next.
%
% If one filename is given, the plots generated are:
%
%     unnormalized magnitude of phasors versus frequency
%     normalized magnitude versus frequency
%     phase relative to 1st channel versus frequency
%
% If two files are specified the plots generated are:
%
%     ratio of unnormalized magnitudes versus frequency (file2/file1)
%     ratio of normalized magnitudes versus frequency   (file2/file1)
%     difference of (phase relative to 1st channel) versus frequency
%                                                       (file2-file1)
%
% Inputs:
% filename1       data file name (without file extension),
%                 must be a string          
% filename2       data file name (without file extension),
%                 must be a string          
%  
% Errors are generated if files do not exist in current directory.

% Check to see if filename1.mat exists in the current directory
path1=[cd '\' filename1 '.mat'];
resultcode=exist(path1,'file'); 
if resultcode ~= 2;  % if not= 2 then file is not present
   error([filename1 '.mat cannot be found in current directory']);
else
   % If filename1.mat exists, load the structure. 
   % The load function below will return 
   %   datstruct.caldata.fields or datstruct.phasordata.fields
   % The lines below convert these structures to data1.fields
   datstruct=load(path1);
   namecell=fieldnames(datstruct);
   data1=eval(['datstruct.' namecell{1}]);
end

% Compute unnormalized magnitude at each frequency.
unnormmags1=abs(data1.phasors);

% Normalize complex amplitudes to 1st channel.  This sets the 1st channel 
% phasor to 1.0 at each frequency, thus removing level variations common
% to all channels.
normdata1=...
    data1.phasors ./ (repmat(data1.phasors(:,1),1,size(data1.phasors,2)));  

% Compute magnitude and phase at each frequency relative to 1st channel
normmags1=abs(normdata1);
phasedata1=unwrap(angle(normdata1));

% Case of one input argument
if nargin==1   

% Create plots
% For linear frequency axis, change 'semilogx' to 'plot'
figure('Name',['View data file: ' filename1 '.mat'])

subplot(3,1,1), semilogx(data1.freqs,unnormmags1)
xlabel('frequency (Hz)'), grid on
ylabel('unnormed mag')

subplot(3,1,2), semilogx(data1.freqs,normmags1)
xlabel('frequency (Hz)'), grid on
ylabel('normed mag')

subplot(3,1,3), semilogx(data1.freqs,phasedata1)
xlabel('frequency (Hz)'), grid on
ylabel('phase (rads)')
    
% Case of two input arguments
elseif nargin==2
    
% Check to see if filename2.mat exists in the current directory
path2=[cd '\' filename2 '.mat'];
resultcode=exist(path2,'file'); 
if resultcode ~= 2;  % if not= 2 then file is not present
   error([filename2 '.mat cannot be found in current directory']);
else
   % If filename2.mat exists, load the structure. 
   % The load function below will return 
   %   datstruct.caldata.fields or datstruct.phasordata.fields
   % The lines below convert these structures to data2.fields
   datstruct=load(path2);
   namecell=fieldnames(datstruct);
   data2=eval(['datstruct.' namecell{1}]);
end

% Check if the two data files can be compared.  Require that
% they have the same number of frequencies and the same number of channels.
if any(size(data1.phasors) ~= size(data2.phasors))
    error('Data files are not of the same size and cannot be compared.')
% Also require that corresponding elements of the frequency lists are the
% equal to an accuracy of 0.01 Hz.
elseif any(abs(data1.freqs-data2.freqs) >= 0.01)
    error('Frequency lists are not the same so files cannot be compared.')
end

% Compute unnormalized magnitude at each frequency.
unnormmags2=abs(data2.phasors);

% Normalize complex amplitudes to 1st channel.  This sets the 1st channel 
% phasor to 1.0 at each frequency, thus removing level variations in 
% the cal data.
normdata2=...
    data2.phasors ./ (repmat(data2.phasors(:,1),1,size(data2.phasors,2)));  

% Compute magnitude and phase at each frequency relative to 1st channel
normmags2=abs(normdata2);
phasedata2=unwrap(angle(normdata2));

% Create plots
% For linear frequency axis, change 'semilogx' to 'plot'
figure('Name',...
    ['Compare data files: ' filename1 '.mat, ' filename2 '.mat'])

subplot(3,1,1), semilogx(data1.freqs,unnormmags2./unnormmags1)
xlabel('frequency (Hz)'), grid on
ylabel('unnormed mag2/unnormed mag1')

subplot(3,1,2), semilogx(data1.freqs,normmags2./normmags1)
xlabel('frequency (Hz)'), grid on
ylabel('normed mag2/normed mag1')

subplot(3,1,3), semilogx(data1.freqs,phasedata2-phasedata1)
xlabel('frequency (Hz)'), grid on
ylabel('phase2-phase1 (rads)')
    
else
    error('viewdata takes 1 or 2 arguments')
end
