function viewwaves(wavefile)
% viewwaves(wavefile) creates plots for inspecting files containing
% the wave data structure, as generated by wavefit.m.  wavefile is 
% the input file name (a string), which must be present in the 
% current directory.  To compare the fit phasors in the wave
% data structure with the experimentally measured phasors that were
% used to generate the fit, viewwaves also opens the datafile 
% containing the phasordata structure.  This file must be present 
% in the current directory.  The name of the file containing the 
% phasordata structure is taken from the field wave.datafile.
%
% The plots generated are:
%
%     magnitude of left and right going wave amplitudes versus frequency
%     phase velocity and attenuation versus frequency
%     normalized chi-square for each channel versus frequency
%     polar plot of data phasors and fit phasors at first frequency
%
% After all plots are displayed the user is prompted to type '+' or '-'
% to scan up or down through the frequency list.  Each time the
% frequency is changed the polar plot is updated.  Hold the enter key
% down to scan rapidly in the direction of the most recent change, and
% type 'q' to quit.
%
% Inputs:
% wavefile        input file name (without file extension),
%                 must be a string          
%  
% Errors are generated if input files do not exist in current directory.
%
% Uses: wavefcnlocal

% Check to see if wavefile.mat exists in the current directory
path1=[cd '\' wavefile '.mat'];
resultcode=exist(path1,'file'); 
if resultcode ~= 2;  % if not= 2 then file is not present
   error([wavefile '.mat cannot be found in current directory']);
else            % load the wave structure into current workspace
   load(path1);
end

% Check to see if datafile.mat containing the phasordata structure
% exists in the current directory
path2=[cd '\' wave.datafile '.mat'];
resultcode=exist(path2,'file'); 
if resultcode ~= 2;  % if not= 2 then file is not present
   error([wave.datafile '.mat cannot be found in current directory']);
else            % load the phasordata structure into workspace
   load(path2);
end

% Plot magnitude of left and right going wave amplitudes
figure('Name',['Left and right going wave magnitude: ' wavefile '.mat'])
subplot(2,1,1), plot( wave.freqs,abs(wave.Ramp) )     % plot right amplitude
xlabel('frequency (Hz)'), grid on
ylabel('right magnitude')
subplot(2,1,2), plot( wave.freqs,abs(wave.Lamp) )  % plot left amplitude
xlabel('frequency (Hz)'), grid on
ylabel('left magitude')

% Plot phase velocity and attenuation
figure('Name',['Phase velocity and attenuation: ' wavefile '.mat'])
subplot(2,1,1), plot(wave.freqs,wave.vphase)   % plot phase velocity
xlabel('frequency (Hz)'), grid on
ylabel('phase velocity (m/s)')
subplot(2,1,2), plot(wave.freqs,wave.atten)  % plot attenuation
xlabel('frequency (Hz)'), grid on
ylabel('attenuation (1/m)')
limits=axis;            % get current axis limits
location=[limits(1)+(limits(2)-limits(1))/10,limits(3)+(limits(4)-limits(3))/10];
% Create string containing wave.global results
globalstring=['method=' wave.method];
for index=1:size(wave.global,2)/2;
    globalstring=[globalstring '  ' wave.global{2*index-1}...
                           '=' num2str(wave.global{2*index})];
end
% Write string to attenuation plot
text(location(1),location(2),globalstring);

% Plot normalized chi-squared versus frequency
figure('Name',...
    ['Normalized chi-squared: ' wavefile '.mat and ' wave.datafile '.mat'])                  
% form residual matrix = (fit phasors) - (experimentally measured phasors)
residual=wave.phasors-phasordata.phasors;   
sqresid=abs(residual).^2;    % complex magnitude squared of each element
% normalize to right going amplitude at each frequency
normmat=repmat(abs(wave.Ramp).^2,1,wave.params.nchan);
chisqnorm=sqresid./normmat;
plot(wave.freqs,chisqnorm)
xlabel('frequency (Hz)'), grid on
ylabel('chi-squared')

% Plot the fit phasors and the experimentally measured phasors on the
% complex plane at each frequency.  Also plot phasors calculated from the
% fit parameters at a dense set of microphone locations.
%
% Get matrix of fit phasors at a dense set of locations.  First must
% repack fit results in a format suitable for wavefcnlocal.m.  See
% wavefit.m comments for details.
fitparams=...
    [real(wave.Ramp);imag(wave.Ramp);real(wave.Lamp);imag(wave.Lamp);...
     wave.vphase;wave.atten];

dense=wave.params.micoffset+wave.params.micscale*(0:1/100:1);
densephasors=wavefcnlocal(fitparams,wave.freqs,dense);

% Initialization for loop
index=1;                         % start index to wave.freqs at 1
toggle=+1;                          % +1 means increment frequency
                                 % -1 means decrement
nfreq=size(wave.freqs,1);         % number of frequencies

% Create figure and get handle
handle=figure('Name',...
 ['Fit and measured phasors: ' wavefile '.mat and ' wave.datafile '.mat']);                  

% start loop, continue until 'q' is typed
flag = true;   
while flag == true
    
    figure(handle)     % make phasor figure current
    % plot measured phasors at each mic location with a black 'x'
    plot(phasordata.phasors(index,:),'kx')   
    axis equal 
    hold on            % stay on same plot
    
    % plot fit phasors at mic locations with a red 'O'
    plot(wave.phasors(index,:),'rO')  
    
    % plot fit phasors at a dense set of points with a red line
    plot(densephasors(index,:,1),densephasors(index,:,2),'r-')  
    
    % label plot with the current frequency
    limits=axis;      % get current axis limits
    location=[limits(1)+(limits(2)-limits(1))/10,limits(3)+(limits(4)-limits(3))/10];
    text(location(1),location(2),[num2str(wave.freqs(index)) ' Hz'])  
    xlabel('complex amplitudes'), grid on
    
    hold off           % new plot next time around
    
    % let user increment or decrement the frequency
    cmd = input('Type ''+'', ''-'' or ''q'' to quit:','s')';
    if strcmp(cmd,'+')     % increment frequency
        if index < nfreq 
            index=index+1;
            toggle=+1;
        end
    elseif strcmp(cmd,'-') % decrement frequency
        if index > 1 
            index=index-1;
            toggle=-1;
        end
    elseif strcmp(cmd,'')  % user hits return without typing anything
        if (toggle > 0) && (index < nfreq) 
            index=index+1; % keep going in same direction as last change
        elseif (toggle < 0) && (index > 1)
            index=index-1;
        end
    elseif strcmp(cmd,'q')  % user wants to quit
        return
    else
        display('Command not recognized');
    end
    
end

