function wave=wavefit(params,method,datafile,wavefile)
% wavefit(params,method,datafile,wavefile) fits right and left going
% wave amplitudes and the propagation constant to the phasor data in the
% input file datafile.  The output data structure wave is written to
% the file wavefile.  Several fitting methods are implemented
% and these are selected by the input string variable named method.  In
% all methods, no constraint is placed on the values of the right
% to left going wave amplitudes and no relation between them is assumed.
%
% Inputs:
% params          parameters data structure, see defaults.m for details
% method          string, selects fitting method
%                 'local'   fits the phase velocity and attenuation
%                           independently at each frequency
%                 '3param'  fits three global parameters that describe the
%                           propagation constant at all frequencies
%                 '2param'  fits two global parameters that describe
%                           the propagation constant at all frequencies
% datafile        string, input file name containing the structure 
%                           phasordata as generated by acquire.m
% wavefile        string, output file name for the structure wave
%
% Output:
% wave.Ramp            Complex wave amplitude for the right-going wave,
%                      a column vector with one entry for each frequency.
% wave.Lamp            Complex wave amplitude for the left-going wave,
%                      a column vector with one entry for each frequency.
% wave.vphase          Column vector containing the phase velocity
%                      at each frequency.  In some methods it is fit 
%                      independently at each frequency, in others it is 
%                      fixed or calculated from a number of 
%                      fit quantities.
% wave.atten           Column vector containing the attenuation constant
%                      at each frequency.  In some methods it is fit 
%                      independently at each frequency, in others it is 
%                      calculated from a number of fit quantities.
% wave.phasors         The fit phasors.  The fitting routine adjusts the
%                      wave amplitudes and propagation parameters so that
%                      the fit phasors are as close as possible to the
%                      data phasors in the input structure phasordata.
%                      The number of rows is equal to the number of
%                      frequencies and there is one column for each
%                      channel.  This is the same format as in
%                      phasordata.phasors.
% wave.global          A cell array containing quantities that are fit
%                      along with the wave amplitudes.  These quantities
%                      are global in the sense that they apply to
%                      all frequencies.  The format is:
%                      {'name1',value1,'name2',value2,...}  There may be
%                      any number of name, value pairs.
% wave.freqs           Column vector of frequencies, same as input
%                      frequencies in phasordata.freqs
% wave.params          The parameters data structure input to wavefit
% wave.datafile        A string, the name of the input data file
% wave.method          Fitting method, the input string named method.
%
% The wavefit function writes a file in the current directory
% with the name wavefile.mat that contains the wave data structure.  If
% wavefile.mat already exists in the current directory, wave returns
% without doing anything.  It also returns without doing anything if
% the input file datafile.mat is not present in the current directory,
% or if the input string named method is not recognized.
%
% Uses: wavfcn3, wavfcn2, wavfcnlocal

% Check file names and load datafile
% Check to see if datafile.mat exists in the current directory
datapath=[cd '\' datafile '.mat'];
resultcode=exist(datapath,'file'); 
if resultcode ~= 2;  % if not= 2 then file is not present
    error([datafile '.mat cannot be found in current directory']);
else
    load(datapath)   % if data.mat exists, load the structure 
                     % phasordata into the workspace
end

% Check that output file is not already present in current directory.
outpath=[cd '\' wavefile '.mat'];
resultcode=exist(outpath,'file'); 
if resultcode == 2;  % if = 2 then file is already present
    error([wavefile '.mat already exists in current directory']);
end

% Check that the number of channels in the input data file is the same 
% as the number of channels specified in the parameters structure.  
if phasordata.params.nchan ~= params.nchan
   error('number of channels does not agree with data file');
end

% Get frequency list from input data file
freqs=phasordata.freqs;
nfreq=size(freqs,1);  % find number of frequency points


% Find microphone locations, in meters from fiducial plane
x=params.micoffset+params.micscale*params.miclocs; 

%************************************************************
if strcmp(method,'3param')
% In this fitting method the phase velocity and attenuation are described 
% by three global parameters:
%
% vs        free space sound velocity
% beta      sqrt(omega) coefficient of attenuation
% delta     vs*delta is 1/sqrt(omega) coeff. of phase velocity
%
% atten=beta*sqrt(omega)
% vphase=vs*(1.0-vs*delta./sqrt(omega))
%
% A right-going wave is proportional to
% exp(-i*k*x) where k=(omega/vphase) + i*atten
% 
% On the theory of Benade(1968) (based on Rayleigh), we expect 
% delta=beta=0.00095 (in MKS) for a 25 mm diameter tube in air.  
% The complex wave amplitudes are fit independently at each frequency.
%
% Set starting values for fit
vsSt=342.2;       % starting value for free space speed of sound
betaSt=0.00115;   % starting value for sqrt(omega) coeff. of atten.
deltaSt=0.00115;  % starting value of coeff. of phase vel.
RampSt=0.1*(1+1i)*(ones(nfreq,1));  % starting value for right and left
LampSt=0.1*(1+1i)*(ones(nfreq,1));  % complex amplitudes

% In this method we use the MATLAB fitting routine lsqnonlin, which
% requires a real-valued cost function of real parameters.  We use an
% extra index to represent real (index=1) and imaginary (index=2) parts
% and add the suffix S (for separated) to variable names.
%
% Separate starting wave amplitudes into real and imaginary parts.
RampStS(:,1)=real(RampSt); RampStS(:,2)=imag(RampSt);
LampStS(:,1)=real(LampSt); LampStS(:,2)=imag(LampSt);

% Pack starting parameters into a real column vector
Start=[RampStS(:,1);RampStS(:,2);LampStS(:,1);LampStS(:,2);...
          vsSt;betaSt;deltaSt];

% Separate the input phasor data into real and imaginary parts
phasorsS(:,:,1)=real(phasordata.phasors);
phasorsS(:,:,2)=imag(phasordata.phasors);

% Create handle to the cost function. 
% Adj contains all of the parameters that will be adjusted to minimize 
% the cost function.  The starting value of Adj is the vector Start.
% The anonymous function construction @ allows freqs, x, and phasorsS 
% to be passed to the fitting routine lsquinlin.  wavefcn3.m is a function
% that returns modeled phasors labeled with 3 indices.  The first index 
% runs over the frequencies in freqs, and the second runs over the mic 
% positions in x.  The last index is for real and imaginary parts.  The 
% cost function is the difference between the modeled phasors returned by
% wavefcn3 and the data phasors from phasordata.phasors.  The fitting
% routine lsqnonlin takes care of squaring each element of the cost
% function and summing over all indicies.
% freqs and Adj are columns, but x is a row.
fhandle = @(Adj) wavefcn3(Adj,freqs,x)-phasorsS ;

options = optimset('Display','iter','TolFun',1e-6);
Finish = lsqnonlin(fhandle,Start,[],[],options); % do the fit

% unpack the results   
wave.Ramp=Finish(1:nfreq)+1i*Finish(nfreq+1:2*nfreq);       
wave.Lamp=Finish(2*nfreq+1:3*nfreq)+1i*Finish(3*nfreq+1:4*nfreq);

vs=Finish(4*nfreq+1);        % unpack fitted propagation parameters
beta=Finish(4*nfreq+2);
delta=Finish(4*nfreq+3);

omega=2*pi*freqs;
wave.vphase=vs*(1.0-vs*delta./sqrt(omega)); % compute phase velocity vector
wave.atten=beta*sqrt(omega);                % compute attenuation vector

fitphasorbuff=wavefcn3(Finish,freqs,x);     % get the fit phasors
                                            % and store as complex numbers
wave.phasors=fitphasorbuff(:,:,1) + 1i*fitphasorbuff(:,:,2);

cellarray={'vs', vs,'beta', beta, 'delta', delta}; % create cell array
wave.global=cellarray;       % store fit parameters in wave.global

wave.freqs=freqs;
wave.params=params;
wave.datafile=datafile;
wave.method=method;

% Write wave structure to file wavefile.mat.  
save(wavefile,'wave');

%*************************************************************
elseif strcmp(method,'2param')
% In this fitting method the frequency-independent phase velocity 
% is a fit parameter and and attenuation is described by beta:
%
% vphase    frequency independent sound velocity
% beta      sqrt(omega) coefficient of attenuation
%
% atten=beta*sqrt(omega)
%
% A right-going wave is proportional to
% exp(-i*k*x) where k=(omega/vphase) + i*atten
% 
% On the theory of Benade(1968) (based on Rayleigh), we expect 
% beta=0.00095 (in MKS) for a 25 mm diameter tube in air.  
% The complex wave amplitudes are fit independently at each frequency.
%
% Set starting values for fit
vphaseSt=342.2;   % starting value for speed of sound
betaSt=0.00115;   % starting value for sqrt(omega) coeff. of atten.
RampSt=0.1*(1+1i)*(ones(nfreq,1));  % starting value for right and left
LampSt=0.1*(1+1i)*(ones(nfreq,1));  % complex amplitudes

% In this method we use the MATLAB fitting routine lsqnonlin, which
% requires a real-valued cost function of real parameters.  We use an
% extra index to represent real (index=1) and imaginary (index=2) parts
% and add the suffix S (for separated) to variable names.
%
% Separate starting wave amplitudes into real and imaginary parts.
RampStS(:,1)=real(RampSt); RampStS(:,2)=imag(RampSt);
LampStS(:,1)=real(LampSt); LampStS(:,2)=imag(LampSt);

% Pack starting parameters into a real column vector
Start=[RampStS(:,1);RampStS(:,2);LampStS(:,1);LampStS(:,2);...
          vphaseSt;betaSt];

% Separate the input phasor data into real and imaginary parts
phasorsS(:,:,1)=real(phasordata.phasors);
phasorsS(:,:,2)=imag(phasordata.phasors);

% Create handle to the cost function. 
% Adj contains all of the parameters that will be adjusted to minimize 
% the cost function.  The starting value of Adj is the vector Start.
% The anonymous function construction @ allows freqs, x, and phasorsS 
% to be passed to the fitting routine lsquinlin.  wavefcn3.m is a function
% that returns modeled phasors labeled with 3 indices.  The first index 
% runs over the frequencies in freqs, and the second runs over the mic 
% positions in x.  The last index is for real and imaginary parts.  The 
% cost function is the difference between the modeled phasors returned by
% wavefcn3 and the data phasors from phasordata.phasors.  The fitting
% routine lsqnonlin takes care of squaring each element of the cost
% function and summing over all indicies.
% freqs and Adj are columns, but x is a row.
fhandle = @(Adj) wavefcn2(Adj,freqs,x)-phasorsS ;

options = optimset('Display','iter','TolFun',1e-6);
Finish = lsqnonlin(fhandle,Start,[],[],options); % do the fit

% unpack the results   
wave.Ramp=Finish(1:nfreq)+1i*Finish(nfreq+1:2*nfreq);       
wave.Lamp=Finish(2*nfreq+1:3*nfreq)+1i*Finish(3*nfreq+1:4*nfreq);

vphase=Finish(4*nfreq+1);        % unpack fitted propagation parameters
beta=Finish(4*nfreq+2);

omega=2*pi*freqs;
wave.vphase=vphase*ones(nfreq,1);   % constant phase velocity vector
wave.atten=beta*sqrt(omega);        % compute attenuation vector

fitphasorbuff=wavefcn2(Finish,freqs,x);     % get the fit phasors
                                            % and store as complex numbers
wave.phasors=fitphasorbuff(:,:,1) + 1i*fitphasorbuff(:,:,2);

cellarray={'vphase', vphase,'beta', beta}; % create cell array
wave.global=cellarray;       % store fit parameters in wave.global

wave.freqs=freqs;
wave.params=params;
wave.datafile=datafile;
wave.method=method;

% Write wave structure to file wavefile.mat.  
save(wavefile,'wave');

%*************************************************************
elseif strcmp(method,'local')
% In this fitting method the phase velocity and attenuation are fit
% independently at each frequency.  This is an unbiased method but it
% involves many unnecessary parameters since the phase velocity and
% attenuation are known to vary slowly with frequency.
%
% A right-going wave is proportional to
% exp(-i*k*x) where k=(omega/vphase) + i*atten
%
% Set starting values for fit
RampSt=0.1*(1+1i)*(ones(nfreq,1));  % starting value for right and left
LampSt=0.1*(1+1i)*(ones(nfreq,1));  % complex amplitudes
vphaseSt=342.2*ones(nfreq,1);       % start phase velocity at 342.2 m/s
attenSt=zeros(nfreq,1);              % start attenuation at zero

% In this method we use the MATLAB fitting routine lsqnonlin, which
% requires a real-valued cost function of real parameters.  We use an
% extra index to represent real (index=1) and imaginary (index=2) parts
% and add the suffix S (for separated) to variable names.
%
% Separate starting wave amplitudes into real and imaginary parts.
RampStS(:,1)=real(RampSt); RampStS(:,2)=imag(RampSt);
LampStS(:,1)=real(LampSt); LampStS(:,2)=imag(LampSt);

% Pack starting parameters into a real column vector
Start=[RampStS(:,1);RampStS(:,2);LampStS(:,1);LampStS(:,2);...
          vphaseSt;attenSt];

% Separate the input phasor data into real and imaginary parts
phasorsS(:,:,1)=real(phasordata.phasors);
phasorsS(:,:,2)=imag(phasordata.phasors);

% Create handle to the cost function. 
% Adj contains all of the parameters that will be adjusted to minimize 
% the cost function.  The starting value of Adj is the vector Start.
% The anonymous function construction @ allows freqs, x, and phasorsS 
% to be passed to the fitting routine lsquinlin.  wavefcnlocal.m
% returns modeled phasors labeled with 3 indices.  The first index 
% runs over the frequencies in freqs, and the second runs over the mic 
% positions in x.  The last index is for real and imaginary parts.  The 
% cost function is the difference between the modeled phasors returned by
% wavefcn3 and the data phasors from phasordata.phasors.  The fitting
% routine lsqnonlin takes care of squaring each element of the cost
% function and summing over all indicies.
% freqs and Adj are columns, but x is a row.
fhandle = @(Adj) wavefcnlocal(Adj,freqs,x)-phasorsS ;

options = optimset('Display','iter','TolFun',1e-6);
Finish = lsqnonlin(fhandle,Start,[],[],options); % do the fit

% unpack the results   
wave.Ramp=Finish(1:nfreq)+1i*Finish(nfreq+1:2*nfreq);       
wave.Lamp=Finish(2*nfreq+1:3*nfreq)+1i*Finish(3*nfreq+1:4*nfreq);
wave.vphase=Finish(4*nfreq+1:5*nfreq);      % get phase velocity vector
wave.atten=Finish(5*nfreq+1:6*nfreq);       % get attenuation vector

fitphasorbuff=wavefcnlocal(Finish,freqs,x); % get the fit phasors
                                            % and store as complex numbers
wave.phasors=fitphasorbuff(:,:,1) + 1i*fitphasorbuff(:,:,2);

cellarray={};             % create empty cell array
wave.global=cellarray;    % no fit parameters to store in wave.global

wave.freqs=freqs;
wave.params=params;
wave.datafile=datafile;
wave.method=method;

% Write wave structure to file wavefile.mat.  
save(wavefile,'wave');

%*************************************************************
else
    error('The method was not recognized.')
end